@@ -61,6 +61,32 @@ StatePost::const_iterator Delta::epsilon_symbol_posts(const StatePost& state_pos
     return state_post.end();
 }
 
+StateSet StatePost::get_successors() const {
+    StateSet successors;
+    for (const SymbolPost& symbol_post: *this) {
+        successors.insert(symbol_post.targets);
+    }
+    return successors;
+}
+
+const StateSet& StatePost::get_successors(Symbol symbol) const {
+    const auto symbol_post_it = find(symbol);
+    if (symbol_post_it == this->end()) {
+        static StateSet empty_set{};
+        return empty_set;
+    }
+    return symbol_post_it->targets;
+}
+
+
+StateSet Delta::get_successors(const State state) const {
+    return state_post(state).get_successors();
+}
+
+const StateSet& Delta::get_successors(State state, Symbol symbol) const {
+    return state_post(state).get_successors(symbol);
+}
+
 std::vector<Transition> Delta::get_transitions_to(const State state_to) const {
     std::vector<Transition> transitions_to_state{};
     const size_t num_of_states{ this->num_of_states() };
@@ -75,22 +101,30 @@ std::vector<Transition> Delta::get_transitions_to(const State state_to) const {
     return transitions_to_state;
 }
 
-void Delta::add(State source, Symbol symbol, State target) {
-    const State max_state{ std::max(source, target) };
-    if (max_state >= state_posts_.size()) {
+std::vector<Transition> Delta::get_transitions_between(const State state_from, const State state_to) const {
+    std::vector<Transition> transitions_between{};
+    for (const SymbolPost& symbol_post: state_post(state_from)) {
+        const StateSet::const_iterator state_to_find_it = symbol_post.targets.find(state_to);
+        if (state_to_find_it != symbol_post.targets.end()) {
+            transitions_between.push_back({ state_from, symbol_post.symbol, state_to });
+        }
+    }
+    return transitions_between;
+}
+
+void Delta::add(const State source, Symbol symbol, const State target) {
+    if (const State max_state{ std::max(source, target) }; max_state >= state_posts_.size()) {
         reserve_on_insert(state_posts_, max_state);
         state_posts_.resize(max_state + 1);
     }
 
-    StatePost& state_transitions{ state_posts_[source] };
-
-    if (state_transitions.empty()) {
+    if (StatePost& state_transitions{ state_posts_[source] }; state_transitions.empty()) {
         state_transitions.insert({ symbol, target });
     } else if (state_transitions.back().symbol < symbol) {
         state_transitions.insert({ symbol, target });
     } else {
-        const auto symbol_transitions{ state_transitions.find(SymbolPost{ symbol }) };
-        if (symbol_transitions != state_transitions.end()) {
+        if (const auto symbol_transitions{ state_transitions.find(SymbolPost{ symbol }) };
+            symbol_transitions != state_transitions.end()) {
             // Add transition with symbol already used on transitions from state_from.
             symbol_transitions->insert(target);
         } else {
@@ -102,25 +136,20 @@ void Delta::add(State source, Symbol symbol, State target) {
 }
 
 void Delta::add(const State source, const Symbol symbol, const StateSet& targets) {
-    if(targets.empty()) {
-        return;
-    }
+    if(targets.empty()) { return; }
 
-    const State max_state{ std::max(source, targets.back()) };
-    if (max_state >= state_posts_.size()) {
+    if (const State max_state{ std::max(source, targets.back()) }; max_state >= state_posts_.size()) {
         reserve_on_insert(state_posts_, max_state + 1);
         state_posts_.resize(max_state + 1);
     }
 
-    StatePost& state_transitions{ state_posts_[source] };
-
-    if (state_transitions.empty()) {
+    if (StatePost& state_transitions{ state_posts_[source] }; state_transitions.empty()) {
         state_transitions.insert({ symbol, targets });
     } else if (state_transitions.back().symbol < symbol) {
         state_transitions.insert({ symbol, targets });
     } else {
-        const auto symbol_transitions{ state_transitions.find(symbol) };
-        if (symbol_transitions != state_transitions.end()) {
+        if (const auto symbol_transitions{ state_transitions.find(symbol) };
+            symbol_transitions != state_transitions.end()) {
             // Add transition with symbolOnTransition already used on transitions from state_from.
             symbol_transitions->insert(targets);
 
@@ -132,54 +161,45 @@ void Delta::add(const State source, const Symbol symbol, const StateSet& targets
     }
 }
 
-void Delta::remove(State src, Symbol symb, State tgt) {
-    if (src >= state_posts_.size()) {
-        return;
-    }
+void Delta::remove(const State source, const Symbol symbol, const State target) {
+    if (source >= state_posts_.size()) { return; }
 
-    StatePost& state_transitions{ state_posts_[src] };
-    if (state_transitions.empty()) {
+    if (StatePost& state_transitions{ state_posts_[source] }; state_transitions.empty()) {
         throw std::invalid_argument(
-                "Transition [" + std::to_string(src) + ", " + std::to_string(symb) + ", " +
-                std::to_string(tgt) + "] does not exist.");
-    } else if (state_transitions.back().symbol < symb) {
+                "Transition [" + std::to_string(source) + ", " + std::to_string(symbol) + ", " +
+                std::to_string(target) + "] does not exist.");
+    } else if (state_transitions.back().symbol < symbol) {
         throw std::invalid_argument(
-                "Transition [" + std::to_string(src) + ", " + std::to_string(symb) + ", " +
-                std::to_string(tgt) + "] does not exist.");
+                "Transition [" + std::to_string(source) + ", " + std::to_string(symbol) + ", " +
+                std::to_string(target) + "] does not exist.");
     } else {
-        const auto symbol_transitions{ state_transitions.find(symb) };
-        if (symbol_transitions == state_transitions.end()) {
+        if (const auto symbol_transitions{ state_transitions.find(symbol) };
+            symbol_transitions == state_transitions.end()) {
             throw std::invalid_argument(
-                    "Transition [" + std::to_string(src) + ", " + std::to_string(symb) + ", " +
-                    std::to_string(tgt) + "] does not exist.");
+                    "Transition [" + std::to_string(source) + ", " + std::to_string(symbol) + ", " +
+                    std::to_string(target) + "] does not exist.");
         } else {
-            symbol_transitions->erase(tgt);
+            symbol_transitions->erase(target);
             if (symbol_transitions->empty()) {
-                state_posts_[src].erase(*symbol_transitions);
+                state_posts_[source].erase(*symbol_transitions);
             }
         }
     }
 }
 
-bool Delta::contains(State src, Symbol symb, State tgt) const
+bool Delta::contains(const State source, const Symbol symbol, const State target) const
 { // {{{
-    if (state_posts_.empty()) {
-        return false;
-    }
+    if (state_posts_.empty()) { return false; }
+    if (state_posts_.size() <= source) { return false; }
 
-    if (state_posts_.size() <= src)
-        return false;
-
-    const StatePost& tl = state_posts_[src];
-    if (tl.empty()) {
-        return false;
-    }
-    auto symbol_transitions{ tl.find(SymbolPost{ symb} ) };
+    const StatePost& tl = state_posts_[source];
+    if (tl.empty()) { return false; }
+    const auto symbol_transitions{ tl.find(SymbolPost{ symbol} ) };
     if (symbol_transitions == tl.cend()) {
         return false;
     }
 
-    return symbol_transitions->targets.find(tgt) != symbol_transitions->targets.end();
+    return symbol_transitions->targets.find(target) != symbol_transitions->targets.end();
 }
 
 bool Delta::contains(const Transition& transition) const {
@@ -197,10 +217,7 @@ size_t Delta::num_of_transitions() const {
 }
 
 bool Delta::empty() const {
-    for (const StatePost& state_post: state_posts_) {
-        if (!state_post.empty()) { return false; }
-    }
-    return true;
+    return std::ranges::all_of(state_posts_, [](const StatePost& state_post) { return state_post.empty(); });
 }
 
 Delta::Transitions::const_iterator::const_iterator(const Delta& delta): delta_{ &delta } {
@@ -557,19 +574,26 @@ OrdVector<Symbol> Delta::get_used_symbols() const {
     // return ov;
 
     ///WITH BOOL VECTOR, DIFFERENT VARIANT? (1.9s):
-    //std::vector<bool> bv = get_used_symbols_bv();
-    //std::vector<Symbol> v(std::count(bv.begin(), bv.end(), true));
-    //return utils::OrdVector<Symbol>(v);
+    // std::vector<bool> bv = get_used_symbols_bv();
+    // utils::OrdVector<Symbol> ov{};
+    // ov.reserve(static_cast<size_t>(std::count(bv.begin(), bv.end(), true)));
+    // const size_t bv_size{ bv.size() };
+    // for (Symbol i = 0; i < bv_size; i++) {
+    //     if (bv[i]) {
+    //         ov.push_back(i);
+    //     }
+    // }
+    // return ov;
 
     //WITH CHAR VECTOR (should be the fastest, haven't tried in this branch):
     //BEWARE: failing in one noodlificatoin test ("Simple automata -- epsilon result") ... strange
-    //BoolVector chv = get_used_symbols_chv();
-    //utils::OrdVector<Symbol> ov;
-    //for(Symbol i = 0;i<chv.size();i++)
+    // BoolVector chv = get_used_symbols_chv();
+    // utils::OrdVector<Symbol> ov;
+    // for(Symbol i = 0;i<chv.size();i++)
     //    if (chv[i]) {
     //        ov.push_back(i);
     //    }
-    //return ov;
+    // return ov;
 }
 
 // Other versions, maybe an interesting experiment with speed of data structures.
@@ -615,7 +639,7 @@ std::set<Symbol> Delta::get_used_symbols_set() const {
 mata::utils::SparseSet<Symbol> Delta::get_used_symbols_sps() const {
 #ifdef _STATIC_STRUCTURES_
     //static seems to speed things up a little
-    static utils::SparseSet<Symbol> symbols(64,false);
+    static utils::SparseSet<Symbol> symbols(64);
     symbols.clear();
 #else
     utils::SparseSet<Symbol> symbols(64);
@@ -644,7 +668,6 @@ std::vector<bool> Delta::get_used_symbols_bv() const {
     for (const StatePost& state_post: state_posts_) {
         for (const SymbolPost& symbol_post: state_post) {
             const size_t capacity{ symbol_post.symbol + 1 };
-            reserve_on_insert(symbols, capacity);
             if (symbols.size() < capacity) {
                 symbols.resize(capacity);
             }
@@ -665,7 +688,10 @@ mata::BoolVector Delta::get_used_symbols_chv() const {
     //symbols.dont_track_elements();
     for (const StatePost& state_post: state_posts_) {
         for (const SymbolPost& symbol_post: state_post) {
-            reserve_on_insert(symbols,symbol_post.symbol);
+            const size_t capacity{ symbol_post.symbol + 1 };
+            if (symbols.size() < capacity) {
+                symbols.resize(capacity * 2);
+            }
             symbols[symbol_post.symbol] = true;
         }
     }
@@ -728,12 +754,16 @@ StateSet SynchronizedExistentialSymbolPostIterator::unify_targets() const {
     return unified_targets;
 }
 
-bool SynchronizedExistentialSymbolPostIterator::synchronize_with(const SymbolPost& sync) {
+bool SynchronizedExistentialSymbolPostIterator::synchronize_with(const Symbol sync_symbol) {
     do {
         if (is_synchronized()) {
             auto current_min_symbol_post_it = get_current_minimum();
-            if (*current_min_symbol_post_it >= sync) { break; }
+            if (current_min_symbol_post_it->symbol >= sync_symbol) { break; }
         }
     } while (advance());
-    return is_synchronized() && *get_current_minimum() == sync;
+    return is_synchronized() && get_current_minimum()->symbol == sync_symbol;
+}
+
+bool SynchronizedExistentialSymbolPostIterator::synchronize_with(const SymbolPost& sync) {
+    return synchronize_with(sync.symbol);
 }
